package ru.dbotthepony.kstarbound.network.packets.serverbound

import kotlinx.coroutines.launch
import ru.dbotthepony.kommons.io.readCollection
import ru.dbotthepony.kommons.io.readVector2i
import ru.dbotthepony.kommons.io.readVector3i
import ru.dbotthepony.kommons.io.writeCollection
import ru.dbotthepony.kommons.io.writeStruct2i
import ru.dbotthepony.kommons.io.writeStruct3i
import ru.dbotthepony.kommons.util.Either
import ru.dbotthepony.kommons.vector.Vector2i
import ru.dbotthepony.kommons.vector.Vector3i
import ru.dbotthepony.kstarbound.defs.world.CelestialParameters
import ru.dbotthepony.kstarbound.io.readEither
import ru.dbotthepony.kstarbound.io.writeEither
import ru.dbotthepony.kstarbound.network.IServerPacket
import ru.dbotthepony.kstarbound.network.packets.clientbound.CelestialResponsePacket
import ru.dbotthepony.kstarbound.server.ServerConnection
import ru.dbotthepony.kstarbound.world.UniversePos
import java.io.DataInputStream
import java.io.DataOutputStream

class CelestialRequestPacket(val requests: Collection<Either<Vector2i, Vector3i>>) : IServerPacket {
	constructor(stream: DataInputStream, isLegacy: Boolean) : this(stream.readCollection { readEither(isLegacy, { readVector2i() }, { readVector3i() }) })

	override fun write(stream: DataOutputStream, isLegacy: Boolean) {
		stream.writeCollection(requests) {
			writeEither(it, isLegacy, { writeStruct2i(it) }, { writeStruct3i(it) })
		}
	}

	override fun play(connection: ServerConnection) {
		connection.scope.launch {
			val responses = ArrayList<Either<CelestialResponsePacket.ChunkData, CelestialResponsePacket.SystemData>>()

			for (request in requests) {
				if (request.isLeft) {
					val chunkPos = request.left()
					responses.add(Either.left(connection.server.universe.getChunk(chunkPos)?.toNetwork() ?: continue))
				} else {
					val systemPos = UniversePos(request.right())
					val map = HashMap<Int, CelestialResponsePacket.PlanetData>()

					for (planet in connection.server.universe.children(systemPos)) {
						val planetData = connection.server.universe.parameters(planet) ?: continue
						val children = HashMap<Int, CelestialParameters>()

						for (satellite in connection.server.universe.children(planet)) {
							children[satellite.satelliteOrbit] = connection.server.universe.parameters(satellite) ?: continue
						}

						map[planet.planetOrbit] = CelestialResponsePacket.PlanetData(planetData, children)
					}

					responses.add(Either.right(CelestialResponsePacket.SystemData(systemPos.location, map)))
				}
			}

			connection.send(CelestialResponsePacket(responses))
		}
	}
}